"""A utility module for plotting audio signals"""
# pylint: disable=C0103
import pydub


try:
    import matplotlib.pyplot as plt
    import numpy as np

    has_plotting = True
except ImportError:
    has_plotting = False


def plot_segment(segment: pydub.AudioSegment) -> None:
    """It plots the amplitude of a segment over time"""
    if not has_plotting:
        message = "parrot1[plotting] must be installed"
        raise ValueError(message)

    T = segment.duration_seconds
    f = segment.frame_rate

    t = np.arange(0, T, 1 / f)
    x = segment.get_array_of_samples()

    plt.plot(t, x)

    plt.xlabel("time [s]")
    plt.ylabel("amplitude")

    plt.show()



import asyncio
import logging
from typing import Any, List, Optional

from openai import AsyncClient
from tqdm.asyncio import tqdm_asyncio as tqdm

from parrot1.commons.generative.base import BaseLLMModel


class OpenaiGPTModel(BaseLLMModel):
    def __init__(self, model_size_or_type: str, client_or_model: Optional[Any] = None):
        super().__init__(model_size_or_type, client_or_model)
        self.__logger = logging.getLogger("OpenaiGPT")
        if self.client_or_model is None:
            self.client_or_model = AsyncClient()

    async def agenerate(self, prompt: str, **kwargs) -> str:
        response = await self.client_or_model.completions.create(
            model=self.model_size_or_type, prompt=prompt, **kwargs
        )
        return response.choices[0].text

    def generate(self, prompt: str, **kwargs) -> str:
        raise NotImplementedError("Not implemented. Use the agenerate method instead.")

    def __call__(self, prompt: str, **kwargs) -> str:
        return asyncio.run(asyncio.create_task(self.agenerate(prompt, **kwargs)))

    async def generate_from_prompts(self, prompts: List[str], **kwargs) -> List[str]:
        responses = await tqdm.gather(
            *[
                self.client_or_model.completions.create(
                    model=self.model_size_or_type, prompt=prompt, **kwargs
                )
                for prompt in prompts
            ]
        )

        return [response.choices[0].text for response in responses]



import pathlib as pl
import os
import shutil

PACKAGE_LOCATION = pl.Path(os.path.dirname(__file__))
ROOT_LOCATION = PACKAGE_LOCATION.parent

USER_HOME = pl.Path.home()
PARROT_DATA_FOLDER = USER_HOME / ".parrot1"
PARROT_CACHED_MODELS = PARROT_DATA_FOLDER / "cached_models"
PARROT_CONFIG_FILE = PARROT_DATA_FOLDER / "config.json"

RESOURCES_LOCATION = PACKAGE_LOCATION / "resources"
DEFAULT_CONFIGS_PATH = RESOURCES_LOCATION / "default_configs.json"

os.makedirs(PARROT_DATA_FOLDER, exist_ok=True)

if not os.path.exists(PARROT_CONFIG_FILE):
    shutil.copyfile(src=DEFAULT_CONFIGS_PATH, dst=PARROT_CONFIG_FILE)



from typing import List

from pydantic import BaseModel


class TimedPiece(BaseModel):
    transcript: str
    start: int
    end: int


class SpeakerStamp(TimedPiece):
    speaker: str


class TimedTranscription(BaseModel):
    text: str
    pieces: List[TimedPiece]



"""The command line interface (CLI) PARRoT app"""
import asyncio
import logging
import shutil
from typing import Optional, Annotated

import typer

from parrot1 import PARROT_CONFIG_FILE, DEFAULT_CONFIGS_PATH
from parrot1.audio.transcription.transcribe import transcribe_video_source
from parrot1.recap.recap_generator import generate_final_result
from parrot1.config.config import PARROT_CONFIGS
import pathlib as pl

from parrot1.recap.tasks import ParrotTask

app = typer.Typer()

logging.basicConfig(level=logging.INFO)


@app.command()
def mail(
    video_path: Annotated[str, typer.Argument()],
    transcript: Annotated[Optional[str], typer.Option("--transcript", "-t")] = None,
    output_filepath: Annotated[
        Optional[str], typer.Option("--output_filepath", "-o")
    ] = None,
    use_faster_whisper: Annotated[
        Optional[bool], typer.Option("--faster-whisper", "-fw")
    ] = False,
    use_llama_cpp: Annotated[
        Optional[bool], typer.Option("--llama-cpp", "-lc")
    ] = False,
) -> None:
    """Generates a recap mail for a given meeting. Optionally give a transcript with speakerstamps."""
    typer.secho("*squawk* Writing mail! *squawk*", fg=typer.colors.BRIGHT_GREEN)
    PARROT_CONFIGS.load_configurations()
    transcription_chunks = transcribe_video_source(
        video_path, use_faster_whisper=use_faster_whisper, transcript=transcript
    )
    email = asyncio.run(
        generate_final_result(
            texts=transcription_chunks,
            task=ParrotTask.MAIL,
            use_llama_cpp=use_llama_cpp,
        )
    )

    if output_filepath:
        output_filepath = pl.Path(output_filepath)
    else:
        output_filepath = pl.Path.cwd() / "mail.txt"

    with open(output_filepath, "w") as f:
        f.write(email)


@app.command()
def report(
    video_path: Annotated[str, typer.Argument()],
    transcript: Annotated[Optional[str], typer.Option("--transcript", "-t")] = None,
    output_filepath: Annotated[
        Optional[str], typer.Option("--output-filepath", "-o")
    ] = None,
    use_faster_whisper: Annotated[
        Optional[bool], typer.Option("--faster-whisper", "-fw")
    ] = False,
    use_llama_cpp: Annotated[
        Optional[bool], typer.Option("--llama-cpp", "-lc")
    ] = False,
) -> None:
    """To generate a report from a video recording of a call

    If --transcript is provided, it will align the available speakerstamps
    """
    typer.secho("*squawk* Writing Recap report! *squawk*", fg=typer.colors.BRIGHT_BLUE)
    PARROT_CONFIGS.load_configurations()
    transcription_chunks = transcribe_video_source(
        video_path, use_faster_whisper=use_faster_whisper, transcript=transcript
    )

    recap = asyncio.run(
        generate_final_result(
            texts=transcription_chunks,
            task=ParrotTask.RECAP,
            use_llama_cpp=use_llama_cpp,
        )
    )

    if output_filepath:
        output_filepath = pl.Path(output_filepath)
    else:
        output_filepath = pl.Path.cwd() / "RECAP.md"

    with open(output_filepath, "w") as f:
        f.write(recap)


@app.command()
def reload_configs():
    """Reloads the default configurations inside the .parrot1 folder"""
    import json

    typer.secho("Reload configurations! *sqwuak*", fg=typer.colors.MAGENTA)
    shutil.copyfile(src=DEFAULT_CONFIGS_PATH, dst=PARROT_CONFIG_FILE)
    PARROT_CONFIGS.load_configurations()
    typer.echo(
        f"New Configs are:\n"
        f"{json.dumps(PARROT_CONFIGS.parrot_configs.dict(), indent=1)}"
    )


if __name__ == "__main__":
    app()



import json
import logging

from pydantic.error_wrappers import ValidationError

from parrot1 import PARROT_CONFIG_FILE
from parrot1.config.config_model import ParrotConfigs


class ParrotConfigurationLoader:
    def __init__(self):
        self.parrot_configs = None
        self.__logger = logging.getLogger(__name__)

    def load_configurations(self):
        try:
            with open(PARROT_CONFIG_FILE, "r") as config_file:
                self.parrot_configs = ParrotConfigs(**json.load(config_file))
        except ValidationError as exc:
            self.__logger.error(
                "Something bad happened during the configurations loading."
                "Try running the 'parrot reload-configs' command in order to reset the configurations to a correct formatting."
            )
            self.__logger.error(exc)


PARROT_CONFIGS = ParrotConfigurationLoader()



import enum

from parrot1.commons.models.language import Language
from parrot1.recap import (
    EMAIL_PROMPT_EN,
    EMAIL_PROMPT_IT,
    CHUNK_PROMPT_EN,
    REPORT_PROMPT_IT,
    REPORT_PROMPT_EN,
    CHUNK_PROMPT_IT,
)


class ParrotTask(str, enum.Enum):
    MAIL = "mail"
    RECAP = "recap"
    CHUNK = "chunk"


TASK_TO_PROMPT = {
    Language.EN: {
        ParrotTask.MAIL: EMAIL_PROMPT_EN,
        ParrotTask.RECAP: REPORT_PROMPT_EN,
        ParrotTask.CHUNK: CHUNK_PROMPT_EN,
    },
    Language.IT: {
        ParrotTask.MAIL: EMAIL_PROMPT_IT,
        ParrotTask.RECAP: REPORT_PROMPT_IT,
        ParrotTask.CHUNK: CHUNK_PROMPT_IT,
    },
}


def resolve_prompt_from_task(task: ParrotTask, language: Language = Language.IT) -> str:
    prompts_for_language = TASK_TO_PROMPT.get(language, None)
    if prompts_for_language is None:
        raise KeyError(
            f"Languange {language} not found in dictionary. "
            f"Available languages are {', '.join([e for e in Language])}"
        )
    prompt = prompts_for_language.get(task, None)

    if prompt is None:
        raise KeyError(
            f"No prompt available for such task {task}. "
            f"Available tasks are {', '.join([e for e in ParrotTask])}"
        )
    return prompt



import asyncio
import io
import logging
from typing import Optional, Any, List

from pydub import AudioSegment

from tqdm.asyncio import tqdm_asyncio

from parrot1.audio.transcription.model import TimedTranscription
from parrot1.commons.asr.base import BaseASRModel
from parrot1.commons.models.language import Language
from openai import AsyncClient


class OpenaiWhisper(BaseASRModel):
    def __init__(
        self,
        model_size_or_type: str,
        language: Optional[Language] = Language.IT,
        prompt: Optional[str] = "",
        temperature: Optional[float] = 0.2,
        beam_size: Optional[int] = 5,
        client_or_model: Any = None,
    ):
        super().__init__(
            model_size_or_type,
            language,
            prompt,
            temperature,
            beam_size,
            client_or_model,
        )
        if self.client_or_model is None:
            self.client_or_model = AsyncClient()
        self.__logger = logging.getLogger("OpenaiWhisper")

    async def atranscribe(
        self, audio: AudioSegment, vtt: bool = False
    ) -> TimedTranscription:
        buffer = io.BytesIO()
        buffer.name = "cucciolo.mp3"
        audio.export(buffer, format="mp3")
        self.__logger.debug("Started transcription")
        transcription = await self.client_or_model.audio.transcriptions.create(
            file=buffer,
            model=self.model_size_or_type,
            language=self.language,
            prompt=self.prompt,
            temperature=self.temperature,
        )
        self.__logger.debug("Finished transcription")
        return TimedTranscription(text=transcription.text, pieces=[])

    def transcribe(self, audio: AudioSegment, vtt: bool = False):
        pass

    def transcribe_chunks(
        self, audio_chunks: List[AudioSegment], vtt: bool = False
    ) -> List[TimedTranscription]:
        transcription_chunks = asyncio.run(
            tqdm_asyncio.gather(*[self.atranscribe(ac, vtt=vtt) for ac in audio_chunks])
        )

        return transcription_chunks



"""A utility module for handling Microsoft Stream transcripts"""
# pylint: disable=C0103
import re
from datetime import timedelta

from parrot1.audio.transcription.model import SpeakerStamp
from parrot1.utils.itertools import pairwise


try:
    import docx
    from pytimeparse2 import disable_dateutil, parse

    has_docx = True
except ImportError:
    has_docx = False


def timedelta_to_millis(x: timedelta) -> int:
    """It converts a timedelta to rounded millis"""
    return round(x.total_seconds() * 1000)


def speakerstamps(filepath: str, T: int) -> list[SpeakerStamp]:
    """It converts a Microsoft Stream .docx transcript to speakerstamps

    Parameters
    ----------
    filepath
        The transcript .docx filepath
    T
        The end of the trascript in millis

    Returns
    -------
        The speakerstamps
    """
    if not has_docx:
        message = "parrot1[docx] must be installed"
        raise ValueError(message)

    disable_dateutil()  # It ensures timedeltas over relativedeltas

    document = docx.Document(filepath)

    P = re.compile(
        r"\n(?P<speaker>[A-Za-z]+(?:\s+[A-Za-z]+)*)\s+(?P<timestamp>[0-9]+:[0-9]+)\n"
    )

    def parsing(x: docx.text.paragraph.Paragraph) -> tuple[str, int] | None:
        match = re.search(P, x.text)

        if not match:
            return None

        speaker = match.group("speaker")

        timestamp = match.group("timestamp")
        timestamp = parse(timestamp, as_timedelta=True)
        timestamp = timedelta_to_millis(timestamp)

        return speaker, timestamp

    M = [parsing(x) for x in document.paragraphs]
    M = [x for x in M if x is not None]

    if T < M[-1][1]:
        message = f"The transcript is longer than {T}"
        raise ValueError(message)

    M = M + [(None, T)]
    X = []

    for (speaker, s), (_, e) in pairwise(M):
        x = SpeakerStamp(transcript="", speaker=speaker, start=s, end=e)
        X.append(x)

    return X



# pylint: disable=C0103
"""A utility module for iterable containers"""
import itertools
import typing


T = typing.TypeVar("T")


def pairwise(S: typing.Iterable[T]) -> typing.Iterable[tuple[T, T]]:
    """It pairwise aggregates iterable elements

    .. math::
        S \longmapsto (s_0, s_1), (s_1, s_2), \ldots, (s_{n - 1}, s_n)
    """
    A, B = itertools.tee(S, 2)

    next(B, None)
    return zip(A, B)



import numpy as np
from pydub import AudioSegment


def format_audio_onto_ndarray(audio_segment: AudioSegment) -> np.ndarray:
    return (
        np.frombuffer(audio_segment.raw_data, np.int16).flatten().astype(np.float32)
        / 32768.0
    )



"""A utility module for handling silence in audio signals"""
# pylint: disable=C0103
from collections import deque

import pydub
from pydub.silence import detect_nonsilent


def split_on_silence(
    segment: pydub.AudioSegment,
    min_silence_len: int = 1000,
    silence_thresh: int = -16,
    padding: int | bool = 100,
    seek_step: int = 1,
) -> tuple[list[pydub.AudioSegment], list[tuple[float, float]]]:
    """It splits an audio segment on silent sections

    Parameters
    ----------
    segment
        The original audio segment
    min_silence_len
        The minimum length of silence in millis for a split
    silence_thresh
        The silence threshold in dBFS
    padding
        The amount of silence chunks should be padded with

        It keeps the audio segment from sounding like it is abruptly cut off
    seek_step
        The step size in millis for iterating over the segment

    Returns
    -------
        The audio chunks and split ranges in millis
    """
    T = len(segment)

    if isinstance(padding, bool):
        padding = T if padding else 0

    R = deque(detect_nonsilent(segment, min_silence_len, silence_thresh, seek_step))

    Q = []

    while R:
        x = R.popleft()

        s = max(x[0] - padding, 0)
        e = min(x[1] + padding, T)

        x = (s, e)

        # It merges overlapping padding
        if Q and s < Q[-1][1]:
            s = Q[-1][0]
            e = max(Q[-1][1], e)

            Q[-1] = (s, e)

            continue

        Q.append(x)

    return [segment[s:e] for s, e in Q], Q
